% File src/library/base/man/eigen.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2015 R Core Team
% Distributed under GPL 2 or later

\name{eigen}
\alias{eigen}
\concept{eigen vector}
\concept{eigen value}
\title{Spectral Decomposition of a Matrix}
\description{
  数値(倍精度実数，整数，論理値)又は複素数値行列の固有値と固有ベクトルを
  計算する．
}
\usage{
eigen(x, symmetric, only.values = FALSE, EISPACK = FALSE)
}
\arguments{
  \item{x}{そのスペクトル分解を計算する数値又は複素数値行列，論理値行列は
    数値に強制変換される．}
  \item{symmetric}{もし \code{TRUE} ならば行列は対称(複素数値ならエルミート)行列と
    され，その下三角部分(対角部分を含む)が使われる．
    もし \code{symmetric} が指定されないと行列の対称性が検査される．}
  \item{only.values}{もし \code{TRUE} なら固有値だけが計算され返される．
    さもなければ固有値と固有ベクトルの双方が返される．}
  \item{EISPACK}{論理値．廃止されており無視される．}
}
\details{
  \code{symmetric} が指定されていないと，コードは行列が数値精度のあり得る
  限界内で対称かどうかを決定しようと試みる．値を自分自身で設定するほうが
  より速く安全である．

  固有値を計算するのは大きな行列に対しては時間がかかる部分である．

  行列の固有値分解は実世界の計算機ではエラーを招きやすい：
  完全な解析が Wilkinson (1965) にある．
  望みうる凡ては \code{x} に適当に近い問題への解である．
  実対称行列は実固有値を繰り返し求める代数的な解を持つものの，
  計算された解は固有値の複素共役対を持つ近似した行列の
  それになるかもしれない．
  
  基礎にある LAPACK コードの不成功の結果は正のエラーコード(最も普通には \code{1})
  を持つエラーに至る：これらは FORTRAN コードの詳細な研究によってのみ
  解釈できる．
}
\value{
  \code{x} のスペクトル分解が以下の成分を持つリストとして返される

  \item{values}{\code{x} の \eqn{p} 個の固有値を含むベクトルで，
    減少順にソートされている．非対称ケースで複素数値になる(実行列に対してすら)場合は
    \code{Mod(values)} の大きさ順になる．
    実非対称行列に対してはベクトルは複素共益固有値の対が検出された時だけ
    複素数値になる．
  }
  \item{vectors}{\eqn{p\times p}{p * p} 行列でその列が \code{x} の固有ベクトルを
    含む．もしくは \code{only.values} が \code{TRUE} なら \code{NULL}．
    ベクトルは長さ1に正規化されている．

    固有値ベクトルは定数の曖昧さを除いて定義されていることを思い出そう：
    長さが指定されていても依然として絶対値が1のスカラ(実行列に対しては符号)を
    除いてのみ定義される．
  }

  もし \code{r <- eigen(A)} で \code{V <- r$vectors; lam <- r$values} ならば，
  \deqn{A = V \Lambda V^{-1}}{A = V Lmbd V^(-1)} (数値的なゴミを除き) であり，
  ここで \eqn{\Lambda =}{Lmbd =}\code{diag(lam)}．
}
\source{
  \code{eigen} は LAPACK ルーチンが \code{DSYEVR}, \code{DGEEV},
  \code{ZHEEV} そして \code{ZGEEV} を使う．

  LAPACK は \url{http://www.netlib.org/lapack} にあり，その使用法は参考文献に
  挙げられている．
}
\references{
  Anderson. E. and ten others (1999)
  \emph{LAPACK Users' Guide}.  Third Edition.  SIAM.\cr
  Available on-line at
  \url{http://www.netlib.org/lapack/lug/lapack_lug.html}.

  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
  \emph{The New S Language}.
  Wadsworth & Brooks/Cole.
  Springer-Verlag Lecture Notes in Computer Science \bold{6}.

  Wilkinson, J. H. (1965) \emph{The Algebraic Eigenvalue Problem.}
  Clarendon Press, Oxford.
}

\seealso{
  \code{eigen} の一般化 である \code{\link{svd}}；関連する分解である
  \code{\link{qr}} と \code{\link{chol}}．

  行列の行列式を計算するには \code{\link{qr}} 分解が
  より効率的である：\code{\link{det}}．
}
\examples{
eigen(cbind(c(1,-1), c(-1,1)))
eigen(cbind(c(1,-1), c(-1,1)), symmetric = FALSE)
# 同じ(異なったアルゴリズム)．

eigen(cbind(1, c(1,-1)), only.values = TRUE)
eigen(cbind(-1, 2:1)) # 値を計算
eigen(print(cbind(c(0, 1i), c(-1i, 0)))) # エルミート ==> 実固有値
## 3 x 3:
eigen(cbind( 1, 3:1, 1:3))
eigen(cbind(-1, c(1:2,0), 0:2)) # 複素数値
}
\keyword{algebra}
\keyword{array}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/eigen.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2015 R Core Team
%% Distributed under GPL 2 or later
%
%\name{eigen}
%\alias{eigen}
%\concept{eigen vector}
%\concept{eigen value}
%\title{Spectral Decomposition of a Matrix}
%\description{
%  Computes eigenvalues and eigenvectors of numeric (double, integer,
%  logical) or complex matrices.
%}
%\usage{
%eigen(x, symmetric, only.values = FALSE, EISPACK = FALSE)
%}
%\arguments{
%  \item{x}{a numeric or complex matrix whose spectral decomposition is to
%    be computed.  Logical matrices are coerced to numeric.}
%  \item{symmetric}{if \code{TRUE}, the matrix is assumed to be symmetric
%    (or Hermitian if complex) and only its lower triangle (diagonal
%    included) is used.  If \code{symmetric} is not specified, the matrix
%    is inspected for symmetry.}
%  \item{only.values}{if \code{TRUE}, only the eigenvalues are computed
%    and returned, otherwise both eigenvalues and eigenvectors are
%    returned.}
%  \item{EISPACK}{logical. Defunct and ignored.}
%}
%\details{
%  If \code{symmetric} is unspecified, the code attempts to
%  determine if the matrix is symmetric up to plausible numerical
%  inaccuracies.  It is faster and surer to set the value yourself.
%
%  Computing the eigenvectors is the slow part for large matrices.
%
%  Computing the eigendecomposition of a matrix is subject to errors on a
%  real-world computer: the definitive analysis is Wilkinson (1965).  All
%  you can hope for is a solution to a problem suitably close to
%  \code{x}.  So even though a real asymmetric \code{x} may have an
%  algebraic solution with repeated real eigenvalues, the computed
%  solution may be of a similar matrix with complex conjugate pairs of
%  eigenvalues.
%  
%  Unsuccessful results from the underlying LAPACK code will result in an
%  error giving a positive error code (most often \code{1}): these can
%  only be interpreted by detailed study of the FORTRAN code.
%}
%\value{
%  The spectral decomposition of \code{x} is returned as components of a
%  list with components
%
%  \item{values}{a vector containing the \eqn{p} eigenvalues of \code{x},
%    sorted in \emph{decreasing} order, according to \code{Mod(values)}
%    in the asymmetric case when they might be complex (even for real
%    matrices).  For real asymmetric matrices the vector will be
%    complex only if complex conjugate pairs of eigenvalues are detected.
%  }
%  \item{vectors}{either a \eqn{p\times p}{p * p} matrix whose columns
%    contain the eigenvectors of \code{x}, or \code{NULL} if
%    \code{only.values} is \code{TRUE}.  The vectors are normalized to
%    unit length.
%
%    Recall that the eigenvectors are only defined up to a constant: even
%    when the length is specified they are still only defined up to a
%    scalar of modulus one (the sign for real matrices).
%  }
%
%  If \code{r <- eigen(A)}, and \code{V <- r$vectors; lam <- r$values},
%  then \deqn{A = V \Lambda V^{-1}}{A = V Lmbd V^(-1)} (up to numerical
%  fuzz), where \eqn{\Lambda =}{Lmbd =}\code{diag(lam)}.
%}
%\source{
%  \code{eigen} uses the LAPACK routines \code{DSYEVR}, \code{DGEEV},
%  \code{ZHEEV} and \code{ZGEEV}.
%
%  LAPACK is from \url{http://www.netlib.org/lapack} and its guide is listed
%  in the references.
%}
%\references{
%  Anderson. E. and ten others (1999)
%  \emph{LAPACK Users' Guide}.  Third Edition.  SIAM.\cr
%  Available on-line at
%  \url{http://www.netlib.org/lapack/lug/lapack_lug.html}.
%
%  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
%  \emph{The New S Language}.
%  Wadsworth & Brooks/Cole.
%  Springer-Verlag Lecture Notes in Computer Science \bold{6}.
%
%  Wilkinson, J. H. (1965) \emph{The Algebraic Eigenvalue Problem.}
%  Clarendon Press, Oxford.
%}
%
%\seealso{
%  \code{\link{svd}}, a generalization of \code{eigen}; \code{\link{qr}}, and
%  \code{\link{chol}} for related decompositions.
%
%  To compute the determinant of a matrix, the \code{\link{qr}}
%  decomposition is much more efficient: \code{\link{det}}.
%}
%\examples{
%eigen(cbind(c(1,-1), c(-1,1)))
%eigen(cbind(c(1,-1), c(-1,1)), symmetric = FALSE)
%# same (different algorithm).
%
%eigen(cbind(1, c(1,-1)), only.values = TRUE)
%eigen(cbind(-1, 2:1)) # complex values
%eigen(print(cbind(c(0, 1i), c(-1i, 0)))) # Hermite ==> real Eigenvalues
%## 3 x 3:
%eigen(cbind( 1, 3:1, 1:3))
%eigen(cbind(-1, c(1:2,0), 0:2)) # complex values
%
%}
%\keyword{algebra}
%\keyword{array}
%
\encoding{UTF8}
