% File src/library/base/man/norm.Rd
% Part of the R package, https://www.R-project.org
% Copyright 2010--2015 R Core Team
% Copyright 2010--2012 The R Foundation
% Distributed under GPL 2 or later

\name{norm}
\alias{norm}
\title{Compute the Norm of a Matrix}
\description{
  \code{x} の行列ノルムを LAPCK を使って計算する．
  ノルムの種類は \code{type} の値で決まる
  1ノルム(\code{"O"})，無限大ノルム(\code{"I"})，
  フロベニウスノルム(\code{"F"})，行列要素の絶対値の最大値(\code{"M"})，
  もしくは\dQuote{スペクトル}または \code{"2"}ノルム．
}
\usage{
norm(x, type = c("O", "I", "F", "M", "2"))
}
\arguments{
  \item{x}{数値行列；\CRANpkg{Matrix} のようなパッケージは更に多くの
    \code{norm()} メソッドを定義していることを注意する．}
  \item{type}{文字列，計算するノルムの種類を指示する．
    \describe{
      \item{\code{"O"}, \code{"o"} 又は \code{"1"}}{1ノルムを指示する
	(列和の絶対値の最大値)；}
      \item{\code{"I"} 又は \code{"i"}}{無限大ノルム
	(行和の絶対値の最大値)；}
      \item{\code{"F"} 又は \code{"f"}}{フロベニウスノルムを指示する
        (\code{x} をベクトルとみなしたユークリッドノルム)；}
      \item{\code{"M"} または \code{"m"}}{\code{x} の全ての要素の絶対値の最大値；
        そして}
      \item{\code{"2"}}{\dQuote{スペクトラル}又は2ノルム，
	\code{x} の最大特異値(\code{\link{svd}})．}
    }
    既定値は \code{"O"}．\code{type[1]} の最初の文字だけが使われる．}
}
\details{
  \code{norm()} の \pkg{base} パッケージのメソッドは Lapack 関数 \code{dlange} を
  呼び出す．

  1, 無限大 そして \code{"M"} ノルムはフロベニウスノルムより高速に
  計算できる．

  基礎にある LAPCK コードからの不成功の結果は正のエラーコードを与える：
  これらは FORTRAN コードの詳しい調査によってのみ解釈できる．
}
\value{
  行列ノルムで非負数値．
}
\source{
  \code{norm = "2"} を除いて LAPACK ルーティン \code{DLANGE}．

  LAPACK は \url{http://www.netlib.org/lapack} にある．
}
\references{
  Anderson, E., \emph{et al} (1994).
  \emph{LAPACK User's Guide},
  2nd edition, SIAM, Philadelphia.
}
\seealso{
  (逆)条件数については \code{\link{rcond}}．
}
\examples{
(x1 <- cbind(1, 1:10))
norm(x1)
norm(x1, "I")
norm(x1, "M")
stopifnot(all.equal(norm(x1, "F"),
                    sqrt(sum(x1^2))))

hilbert <- function(n) { i <- 1:n; 1 / outer(i - 1, i, "+") }
h9 <- hilbert(9)
## ノルムの全ての5タイプ：
(nTyp <- eval(formals(base::norm)$type))
sapply(nTyp, norm, x = h9)
}
\keyword{math}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/norm.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 2010--2015 R Core Team
%% Copyright 2010--2012 The R Foundation
%% Distributed under GPL 2 or later
%
%\name{norm}
%\alias{norm}
%\title{Compute the Norm of a Matrix}
%\description{
%  Computes a matrix norm of \code{x} using LAPACK.  The norm can be
%  the one (\code{"O"}) norm, the infinity (\code{"I"}) norm, the
%  Frobenius (\code{"F"}) norm, the maximum modulus (\code{"M"}) among
%  elements of a matrix, or the \dQuote{spectral} or \code{"2"}-norm, as
%  determined by the value of \code{type}.
%}
%\usage{
%norm(x, type = c("O", "I", "F", "M", "2"))
%}
%\arguments{
%  \item{x}{numeric matrix; note that packages such as \CRANpkg{Matrix}
%    define more \code{norm()} methods.}
%  \item{type}{character string, specifying the \emph{type} of matrix
%    norm to be computed.
%    A character indicating the type of norm desired.
%    \describe{
%      \item{\code{"O"}, \code{"o"} or \code{"1"}}{specifies the \bold{o}ne norm,
%	(maximum absolute column sum);}
%      \item{\code{"I"} or \code{"i"}}{specifies the \bold{i}nfinity norm (maximum
%	absolute row sum);}
%      \item{\code{"F"} or \code{"f"}}{specifies the \bold{F}robenius norm (the
%	Euclidean norm of \code{x} treated as if it were a vector);}
%      \item{\code{"M"} or \code{"m"}}{specifies the \bold{m}aximum modulus of
%	all the elements in \code{x}; and}
%      \item{\code{"2"}}{specifies the \dQuote{spectral} or 2-norm, which
%	is the largest singular value (\code{\link{svd}}) of \code{x}.}
%    }
%    The default is \code{"O"}.  Only the first character of
%    \code{type[1]} is used.}
%}
%\details{
%  The \pkg{base} method of \code{norm()} calls the Lapack function
%  \code{dlange}.
%
%  Note that the 1-, Inf- and \code{"M"} norm is faster to calculate than
%  the Frobenius one.
%
%  Unsuccessful results from the underlying LAPACK code will result in an
%  error giving a positive error code: these can only be interpreted by
%  detailed study of the FORTRAN code.
%}
%\value{
%  The matrix norm, a non-negative number.
%}
%\source{
%  Except for \code{norm = "2"}, the LAPACK routine \code{DLANGE}.
%
%  LAPACK is from \url{http://www.netlib.org/lapack}.
%}
%\references{
%  Anderson, E., \emph{et al} (1994).
%  \emph{LAPACK User's Guide},
%  2nd edition, SIAM, Philadelphia.
%}
%\seealso{
%  \code{\link{rcond}} for the (reciprocal) condition number.
%}
%\examples{
%(x1 <- cbind(1, 1:10))
%norm(x1)
%norm(x1, "I")
%norm(x1, "M")
%stopifnot(all.equal(norm(x1, "F"),
%                    sqrt(sum(x1^2))))
%
%hilbert <- function(n) { i <- 1:n; 1 / outer(i - 1, i, "+") }
%h9 <- hilbert(9)
%## all 5 types of norm:
%(nTyp <- eval(formals(base::norm)$type))
%sapply(nTyp, norm, x = h9)
%}
%\keyword{math}
%
\encoding{UTF8}
