% File src/library/methods/man/validObject.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2007 R Core Team
% Distributed under GPL 2 or later

\name{validObject}
%@%\alias{オブジェクトの適正さのテスト}
\alias{validObject}
\alias{getValidity}
\alias{setValidity}
\title{オブジェクトの適正さをテストする}
%\title{ Test the Validity of an Object }
\description{
  \code{object} のそのクラス定義に関する適正さがテストされる．
  もしオブジェクトが適正なら \code{TRUE} が返される；さもなければ
  不適切さを説明する文字列ベクトルが返されるか，
  エラーが生じる(\code{test} が \code{TRUE} かどうかに従って)．
  オプションで，オブジェクト中の全てのスロットがまた検証される．

  関数 \code{setValidity} はクラスの検証メソッドを設定する
  (しかしより普通には，このメソッドは
  \code{\link{setClass}} の \code{validity} 引数として提供される)．
  メソッドは \code{TRUE} か非適切の記述を返す一つのオブジェクトの
  関数であるべきである．
}
\usage{
validObject(object, test = FALSE, complete = FALSE)

setValidity(Class, method, where = topenv(parent.frame()) )

getValidity(ClassDef)
}
\arguments{
  \item{object}{任意のオブジェクトだが，オブジェクトが形式的定義を
    持たない限りあまり何も起きない．}
  \item{test}{
    論理値；もし \code{TRUE} で不適切とされれば，
    この関数は問題を説明する文字列ベクトルを返す．
    もし \code{test} が\code{FALSE} (既定値)ならば正当性検証の
    失敗はエラーを惹き起こす．}
  \item{complete}{論理値；もし \code{TRUE} ならば検証メソッドは
    そうしたメソッドを持つスロット全てに再帰的に適用される．}
  \item{Class}{検証メソッドが設定されるクラスの名前かクラス定義．}
  \item{ClassDef}{クラス定義オブジェクトがで，
    \code{\link{getClassDef}} に由来するようなもの．}
  \item{method}{検証メソッド；つまり，
    \code{NULL} か引数(\code{object})が一つの関数．
    \code{validObject} と同様に，関数はもしオブジェクトが適正であれば
    \code{TRUE} を，もし問題があれば
    一つまたはそれ以上の説明文字列を返すべきである．
    \code{validObject} とは異なり，これはエラーを発生すべきではない．
  }
  \item{where}{修正されたクラス定義はこの環境中に保管される．}

  検証メソッドはスーパークラスの適正さをチェックする必要はないことを
  注意する：\code{validObject} のロジックはこれらのテストが唯一回
  行われることを保証する．
  結論として，もし一つの検証メソッドが別のを使いたければ，
  それは \code{getValidity()} を呼び出して他のクラスの他の定義から
  メソッドを取り出して呼び出すべきである：それは
  \code{validObject} を呼び出すべきでは\emph{無い}．
}
\details{
  検証テストは\sQuote{ボトムアップ}で行われる：オプションで，
  もし \code{complete=TRUE} ならば，もしあればオブジェクトのスロットの
  適切さがテストされる．
  それから，全てのケースで，このクラスが拡張するクラス
  (\sQuote{スーパークラス})の各々に対して，
  もしあればそのクラスの明示的な検証メソッドが
  呼び出される．最後に，もしあれば，\code{object} のクラスの検証メソッドが
  呼び出される．

  あるスロットがその適正さのテストで失敗しても全てのスロットが
  検査されるものの，
  テストは一般にエラーを見つけた最初の段階で停止する．

  標準の検証テスト(\code{complete=FALSE} を用いる)が
  オブジェクトが任意の追加引数で \code{\link{new}} により
  作られた時適用される
  (余分の引数無しでは結果は単にクラスのプロトタイプオブジェクト
  である)．

  もしその引数が
  \code{object} でなければ検証メソッドの定義をフィックスする
  試みがなされる．
}
\value{
  \code{validObject} はもしオブジェクトが適切ならば \code{TRUE}
  を返す．さもなければ見つかった問題を説明する文字列ベクトルを返すが，
  もし \code{test} が \code{FALSE} ならば適正さの失敗は
  対応するエラーメッセージ文字列を伴うエラーを引き起こす．
}
\references{
 Chambers, John M. (2008)
 \emph{Software for Data Analysis: Programming with R}
  Springer.  (R バージョンに対して．)

 Chambers, John M. (1998)
 \emph{Programming with Data}
 Springer (オリジナルの S4 バージョンに対して．)
}
\seealso{\code{\link{setClass}};
  class \code{\linkS4class{classRepresentation}}.
}
\examples{
setClass("track",
          representation(x="numeric", y = "numeric"))
t1 <- new("track", x=1:10, y=sort(stats::rnorm(10)))
## 適正な "track" オブジェクトは同じ数の x, y 値を持つ
validTrackObject <- function(object) {
    if(length(object@x) == length(object@y)) TRUE
    else paste("Unequal x,y lengths: ", length(object@x), ", ",
               length(object@y), sep="")
}
## 関数をクラスに対する適正さメソッドとして付値する
setValidity("track", validTrackObject)
## t1 は適正な "track" オブジェクトであるべきである
validObject(t1)
## ここで何かまずいことを行う
t2 <- t1
t2@x <- 1:20
## これはエラーを起こすべき
\dontrun{try(validObject(t2))}
\dontshow{stopifnot(is(try(validObject(t2)), "try-error"))}

setClass("trackCurve",
         representation("track", smooth = "numeric"))

## validObject が引数付きの initialize() から呼び出される時
## 全てのスーパークラス適正メソッドが使われるので，これは失敗する
\dontrun{trynew("trackCurve", t2)}
\dontshow{stopifnot(is(try(new("trackCurve", t2)), "try-error"))}

setClass("twoTrack", representation(tr1 = "track", tr2 ="track"))

## 適正さのテストは既定では再帰的に適用されないので，
## このオブジェクトが作られる(不適切に)
tT  <- new("twoTrack", tr2 = t2)

## より厳格なテストは問題を検出する
\dontrun{try(validObject(tT, complete = TRUE))}
\dontshow{stopifnot(is(try(validObject(tT, complete = TRUE)), "try-error"))}
}
\keyword{programming}
\keyword{classes}

\encoding{UTF8}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/methods/man/validObject.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2007 R Core Team
%% Distributed under GPL 2 or later
%
%\name{validObject}
%\alias{validObject}
%\alias{getValidity}
%\alias{setValidity}
%\title{ Test the Validity of an Object }
%\description{
%  The validity of \code{object} related to its class definition is
%  tested.  If the object is valid, \code{TRUE} is returned; otherwise,
%  either a vector of strings describing validity failures is returned,
%  or an error is generated (according to whether \code{test} is
%  \code{TRUE}).  Optionally, all slots in the object can also be validated.
%
%  The function \code{setValidity} sets the validity method of a class
%  (but more normally, this method will be supplied as the
%  \code{validity} argument to \code{\link{setClass}}).  The method
%  should be a function of one object that returns \code{TRUE} or a
%  description of the non-validity.
%}
%\usage{
%validObject(object, test = FALSE, complete = FALSE)
%
%setValidity(Class, method, where = topenv(parent.frame()) )
%
%getValidity(ClassDef)
%}
%\arguments{
%  \item{object}{ any object, but not much will happen unless the
%    object's class has a formal definition.}
%  \item{test}{logical; if \code{TRUE} and validity fails, the
%    function returns a vector of strings describing the problems.  If
%    \code{test} is \code{FALSE} (the default) validity failure generates
%    an error.}
%  \item{complete}{logical; if \code{TRUE}, validity methods will be
%    applied recursively to any of the slots that have such methods.}
%  \item{Class}{the name or class definition of the class whose validity
%    method is to be set.}
%  \item{ClassDef}{a class definition object, as from
%    \code{\link{getClassDef}}.}
%  \item{method}{a validity method;  that is, either \code{NULL} or a
%    function of one argument (\code{object}).  Like
%    \code{validObject}, the function should return \code{TRUE} if the
%    object is valid, and one or more descriptive strings if any problems
%    are found.  Unlike \code{validObject}, it should never generate an
%    error.
%  }
%  \item{where}{the modified class definition will be stored in this
%    environment.}
%
%  Note that validity methods do not have to check validity of
%  superclasses: the logic of \code{validObject} ensures these tests are
%  done once only.  As a consequence, if one validity method wants to use
%  another, it should extract and call the method from the other
%  definition of the other class by calling \code{getValidity()}: it
%  should \emph{not} call \code{validObject}.
%}
%\details{
%  Validity testing takes place \sQuote{bottom up}: Optionally, if
%  \code{complete=TRUE}, the validity of the object's slots, if any, is
%  tested.  Then, in all cases, for each of the classes that this class
%  extends (the \sQuote{superclasses}), the explicit validity method of
%  that class is called, if one exists.  Finally, the validity method of
%  \code{object}'s class is called, if there is one.
%
%  Testing generally stops at the first stage of finding an error, except
%  that all the slots will be examined even if a slot has failed its
%  validity test.
%
%  The standard validity test (with \code{complete=FALSE}) is applied
%  when an object is created via \code{\link{new}} with any optional
%  arguments (without the extra arguments the result is just the class
%  prototype object).
%
%  An attempt is made to fix up the definition of a validity method if
%  its argument is not \code{object}.
%}
%\value{
%  \code{validObject} returns \code{TRUE} if the object is valid.
%  Otherwise a vector of strings describing problems found, except that
%  if \code{test} is \code{FALSE}, validity failure generates an error,
%  with the corresponding strings in the error message.
%}
%\references{
% Chambers, John M. (2008)
% \emph{Software for Data Analysis: Programming with R}
%  Springer.  (For the R version.)
%
% Chambers, John M. (1998)
% \emph{Programming with Data}
% Springer (For the original S4 version.)
%}
%\seealso{\code{\link{setClass}};
%  class \code{\linkS4class{classRepresentation}}.
%}
%\examples{
%setClass("track",
%          representation(x="numeric", y = "numeric"))
%t1 <- new("track", x=1:10, y=sort(stats::rnorm(10)))
%## A valid "track" object has the same number of x, y values
%validTrackObject <- function(object) {
%    if(length(object@x) == length(object@y)) TRUE
%    else paste("Unequal x,y lengths: ", length(object@x), ", ",
%               length(object@y), sep="")
%}
%## assign the function as the validity method for the class
%setValidity("track", validTrackObject)
%## t1 should be a valid "track" object
%validObject(t1)
%## Now we do something bad
%t2 <- t1
%t2@x <- 1:20
%## This should generate an error
%\dontrun{try(validObject(t2))}
%\dontshow{stopifnot(is(try(validObject(t2)), "try-error"))}
%
%setClass("trackCurve",
%         representation("track", smooth = "numeric"))
%
%## all superclass validity methods are used when validObject
%## is called from initialize() with arguments, so this fails
%\dontrun{trynew("trackCurve", t2)}
%\dontshow{stopifnot(is(try(new("trackCurve", t2)), "try-error"))}
%
%setClass("twoTrack", representation(tr1 = "track", tr2 ="track"))
%
%## validity tests are not applied recursively by default,
%## so this object is created (invalidly)
%tT  <- new("twoTrack", tr2 = t2)
%
%## A stricter test detects the problem
%\dontrun{try(validObject(tT, complete = TRUE))}
%\dontshow{stopifnot(is(try(validObject(tT, complete = TRUE)), "try-error"))}
%}
%\keyword{programming}
%\keyword{classes}
%
%\encoding{UTF8}
%