% File src/library/stats/man/simulate.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2014 R Core Team
% Distributed under GPL 2 or later

\name{simulate}
\title{Simulate Responses}
\description{
  当てはめモデルオブジェクトに対応する分布から
  一つ又はそれ以上の応答値をシミュレーションする．
}
\usage{
simulate(object, nsim = 1, seed = NULL, \dots)
}
\alias{simulate}%@%\alias{当てはめモデルからの応答値のシミュレーション}
\arguments{
  \item{object}{当てはめモデルを表すオブジェクト．}
  \item{nsim}{シミュレーションする応答値の数．既定では \code{1}．}
  \item{seed}{
    乱数を初期化するか，またどのように初期化するか(\sQuote{乱数種})を
    指示するオブジェクト．\cr
    "lm" メソッドに対しては \code{NULL} か応答ベクトルのシミュレーション前に
    \code{set.seed} への呼び出し中で使われる整数．
    もし設定されるとその値は返り値の \code{"seed"} 属性に保存される．
    既定の \code{NULL} は乱数生成器の状態を変えず，
    \code{\link{.Random.seed}} を
    \code{"seed"} 属性として返す，\sQuote{値}を見よ．
  }
  \item{\dots}{追加のオプション引数．}
}
\value{
  典型的には 長さ \code{nsim} のシミュレーションした応答値のリスト．
  適当であれば結果はデータフレーム(リストの特殊形)になることが出来る．
  %% a *matrix* seems very natural and is more efficient
  %% for large-scale simulation, already for stats:::simulate.lm (in ../R/lm.R )

  \code{"lm"} メソッドに対しては，結果は
  \code{"seed"} 属性を持つデータフレームである．
  もし引数 \code{seed} が \code{NULL} なら，この属性は
  シミュレーションが始まる前の
  \code{\link{.Random.seed}} の値である；
  さもなければそれは \code{"kind"} 属性を持つ引数の値で
  値 \code{as.list(\link{RNGkind}())} を持つ．
}
\details{
  これは総称的関数である．この関数をどのように使うかの詳細は
  個別のモデリング関数を参照せよ．

  パッケージ \pkg{stats} は
  \code{\link{lm}} と \code{\link{glm}} 当てはめで使われる
  \code{"lm"} に対するメソッドを持つ．
  パッケージ \CRANpkg{MASS} 中には
  \code{glm.nb} による当てはめ用のメソッドがあり，
  従って負の二項分布族は
  \code{"lm"} メソッドではカバーされない．

  \code{lm} や \code{glm(family = "gaussian")} で当てはめられた線形モデルに
  対するメソッドは提供された任意の重みは
  エラー分散の逆数に比例すると仮定される．
  他の GLM に対しては \code{\link{family}} オブジェクトの
  (オプションの) \code{simulate} 成分が使われる
  ---  \sQuote{準}モデルは二つのモーメントだけで指定されるので，
  適当なシミュレーション法は無い．

  二項とポアッソン GLM に対しては散布度は1に固定される．
  整数の事前重み \eqn{w_i} は
  観測値 \eqn{i} が \eqn{w_i} 個の観測値の平均を意味すると解釈することが出来，
  これは割合として指定される二項ケースでは自然であるが，
  ポアッソンケースに対してはそれほどではなく
  警告と共に重みは無視される．

  ガンマ GLM に対しては形状パラメータは最尤法で推定される
  (パッケージ \CRANpkg{MASS} 中の関数
  \code{\link[MASS:gamma.shape.glm]{gamma.shape}} を使う)．
  散布度は形状の逆数に比例するため，
  重みの解釈は基本の形状パラメータの乗数としてである．

  逆ガウシアン GLM に対してはモデルは
  \eqn{IG(\mu_i, \lambda w_i)} (
  \url{https://en.wikipedia.org/wiki/Inverse_Gaussian_distribution} を見よ)
  を仮定する，ここで \eqn{\lambda} は当てはめに対する散布度の
  逆数で推定される．
  分散は \eqn{\mu_i^3/(\lambda w_i)} で，従って事前重みの逆数に
  比例する．
  シミュレーションはパッケージ \CRANpkg{SuppDists} からの関数
  \code{\link[SuppDists:invGauss]{rinvGauss}} で行われ，
  インストールされている必要がある．
}
\seealso{
  \R の乱数生成については code{\link{RNG}}，
  関連するメソッドについては \code{\link{fitted.values}} と \code{\link{residuals}}；
  モデル当てはめに対しては \code{\link{glm}}．

  パッケージ \pkg{stats} に対する \file{simulate.R} テストファイルには
  これ以上の例がある．
}
\examples{
x <- 1:5
mod1 <- lm(c(1:3, 7, 6) ~ x)
S1 <- simulate(mod1, nsim = 4)
## シミュレーションを繰り返す：
.Random.seed <- attr(S1, "seed")
identical(S1, simulate(mod1, nsim = 4))

S2 <- simulate(mod1, nsim = 200, seed = 101)
rowMeans(S2) # 次と約同じでなければならない
fitted(mod1)

## 同じことを繰り返す：
(sseed <- attr(S2, "seed")) #乱数種；RNGkind を属性として
stopifnot(identical(S2, simulate(mod1, nsim = 200, seed = sseed)))

## 正しい RNGkind を確認
RNGversion("2.7.0")
## 最初に RNG の種類を設定し，それからシミュレーション
do.call(RNGkind, attr(sseed, "kind"))
identical(S2, simulate(mod1, nsim = 200, seed = sseed))

## 二項 GLM の例
yb1 <- matrix(c(4, 4, 5, 7, 8, 6, 6, 5, 3, 2), ncol = 2)
modb1 <- glm(yb1 ~ x, family = binomial)
S3 <- simulate(modb1, nsim = 4)
# S3 の各項は二列の行列．

x2 <- sort(runif(100))
yb2 <- rbinom(100, prob = plogis(2*(x2-1)), size = 1)
yb2 <- factor(1 + yb2, labels = c("failure", "success"))
modb2 <- glm(yb2 ~ x2, family = binomial)
S4 <- simulate(modb2, nsim = 4)
# S4 の各項は因子
}
\keyword{models}
\keyword{datagen}

\encoding{UTF-8}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/stats/man/simulate.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2014 R Core Team
%% Distributed under GPL 2 or later
%
%\name{simulate}
%\title{Simulate Responses}
%\description{
%  Simulate one or more responses from the distribution
%  corresponding to a fitted model object.
%}
%\usage{
%simulate(object, nsim = 1, seed = NULL, \dots)
%}
%\alias{simulate}
%\arguments{
%  \item{object}{an object representing a fitted model.}
%  \item{nsim}{number of response vectors to simulate.  Defaults to \code{1}.}
%  \item{seed}{an object specifying if and how the random number
%    generator should be initialized (\sQuote{seeded}).\cr
%    For the "lm" method, either \code{NULL} or an integer that will be
%    used in a call to \code{set.seed} before simulating the response
%    vectors.  If set, the value is saved as the \code{"seed"} attribute
%    of the returned value.  The default, \code{NULL} will not change the
%    random generator state, and return \code{\link{.Random.seed}} as the
%    \code{"seed"} attribute, see \sQuote{Value}.
%  }
%  \item{\dots}{additional optional arguments.}
%}
%\value{
%  Typically, a list of length \code{nsim} of simulated responses.  Where
%  appropriate the result can be a data frame (which is a special type of
%  list).
%  %% a *matrix* seems very natural and is more efficient
%  %% for large-scale simulation, already for stats:::simulate.lm (in ../R/lm.R )
%
%  For the \code{"lm"} method, the result is a data frame with an
%  attribute \code{"seed"}.  If argument \code{seed} is \code{NULL}, the
%  attribute is the value of \code{\link{.Random.seed}} before the
%  simulation was started; otherwise it is the value of the argument with
%  a \code{"kind"} attribute with value \code{as.list(\link{RNGkind}())}.
%}
%\details{
%  This is a generic function.  Consult the individual modeling functions
%  for details on how to use this function.
%
%  Package \pkg{stats} has a method for \code{"lm"} objects which is used
%  for \code{\link{lm}} and \code{\link{glm}} fits.  There is a method
%  for fits from \code{glm.nb} in package \CRANpkg{MASS}, and hence the
%  case of negative binomial families is not covered by the \code{"lm"}
%  method.
%
%  The methods for linear models fitted by \code{lm} or \code{glm(family
%  = "gaussian")} assume that any weights which have been supplied are
%  inversely proportional to the error variance.  For other GLMs the
%  (optional) \code{simulate} component of the \code{\link{family}}
%  object is used---there is no appropriate simulation method for
%  \sQuote{quasi} models as they are specified only up to two moments.
%
%  For binomial and Poisson GLMs the dispersion is fixed at one.  Integer
%  prior weights \eqn{w_i} can be interpreted as meaning that
%  observation \eqn{i} is an average of \eqn{w_i} observations, which is
%  natural for binomials specified as proportions but less so for a
%  Poisson, for which prior weights are ignored with a warning.
%
%  For a gamma GLM the shape parameter is estimated by maximum likelihood
%  (using function \code{\link[MASS:gamma.shape.glm]{gamma.shape}} in package
%  \CRANpkg{MASS}).  The interpretation of weights is as multipliers to a
%  basic shape parameter, since dispersion is inversely proportional to
%  shape.
%
%  For an inverse gaussian GLM the model assumed is
%  \eqn{IG(\mu_i, \lambda w_i)} (see
%  \url{https://en.wikipedia.org/wiki/Inverse_Gaussian_distribution})
%  where \eqn{\lambda} is estimated by the inverse of the dispersion
%  estimate for the fit.  The variance is
%  \eqn{\mu_i^3/(\lambda w_i)} and
%  hence inversely proportional to the prior weights.  The simulation is
%  done by function \code{\link[SuppDists:invGauss]{rinvGauss}} from the
%  \CRANpkg{SuppDists} package, which must be installed.
%}
%\seealso{
%  \code{\link{RNG}} about random number generation in \R,
%  \code{\link{fitted.values}} and \code{\link{residuals}} for related methods;
%  \code{\link{glm}}, \code{\link{lm}} for model fitting.
%
%  There are further examples in the \file{simulate.R} tests file in the
%  sources for package \pkg{stats}.
%}
%\examples{
%x <- 1:5
%mod1 <- lm(c(1:3, 7, 6) ~ x)
%S1 <- simulate(mod1, nsim = 4)
%## repeat the simulation:
%.Random.seed <- attr(S1, "seed")
%identical(S1, simulate(mod1, nsim = 4))
%
%S2 <- simulate(mod1, nsim = 200, seed = 101)
%rowMeans(S2) # should be about the same as
%fitted(mod1)
%
%## repeat identically:
%(sseed <- attr(S2, "seed")) # seed; RNGkind as attribute
%stopifnot(identical(S2, simulate(mod1, nsim = 200, seed = sseed)))
%
%## To be sure about the proper RNGkind, e.g., after
%RNGversion("2.7.0")
%## first set the RNG kind, then simulate
%do.call(RNGkind, attr(sseed, "kind"))
%identical(S2, simulate(mod1, nsim = 200, seed = sseed))
%
%## Binomial GLM examples
%yb1 <- matrix(c(4, 4, 5, 7, 8, 6, 6, 5, 3, 2), ncol = 2)
%modb1 <- glm(yb1 ~ x, family = binomial)
%S3 <- simulate(modb1, nsim = 4)
%# each column of S3 is a two-column matrix.
%
%x2 <- sort(runif(100))
%yb2 <- rbinom(100, prob = plogis(2*(x2-1)), size = 1)
%yb2 <- factor(1 + yb2, labels = c("failure", "success"))
%modb2 <- glm(yb2 ~ x2, family = binomial)
%S4 <- simulate(modb2, nsim = 4)
%# each column of S4 is a factor
%}
%\keyword{models}
%\keyword{datagen}
%
%\encoding{UTF-8}
%
