\name{assertCondition}
%@%\alias{エラー条件を確認}
\alias{assertCondition}
\alias{assertWarning}
\alias{assertError}
\title{エラー条件を確認}
%\title{Asserting Error Conditions}
\description{
  コードのテストでは，結果が正しいことをチェックするだけでは
  不十分で，同時にエラーや警告が適当な状況でシグナルされることが必要である．
  ここで説明される関数はそれを行う便利な機能を提供する．
  この三つの関数は提供された表現式の評価がエラー，警告，又は
  指定された条件のリストの一つを生成するかどうかをそれぞれチェックする．
  もし主張が失敗すれば警告が出る．
}
\usage{
assertError(expr, verbose = FALSE)
assertWarning(expr, verbose = FALSE)
assertCondition(expr, ..., .exprString = , verbose = FALSE)
}
\arguments{
  \item{expr}{
    \code{\link{tryCatch}(expr, ..)} で評価される
    未評価の \R 表現式．}
  \item{\dots}{主張を満足する条件のクラスに対応する文字列，
    例えば \code{"error"} や \code{"warning"}．もし何も指定されなければ，
    任意の条件は主張を満足する．詳細節を見よ．}
  \item{.exprString}{
    \code{expr} に対応するプリントされる文字列．
    既定では，実際の \code{expr} が逆構文解析される．
    もし関数がテストされる実際の表現式と共に提供されると省略される．
    もし \code{assertCondition()} が別の関数から呼び出され，
    実際の表現式がその関数への引数として渡されると，
    逆構文解析されたバージョンを提供する．}
  \item{verbose}{もし \code{TRUE} ならば，条件が満足されるとメッセージがプリントされる．}
}
\details{
  \code{assertCondition()} は
  \code{expr} の評価中で生成される全ての条件をチェックするために
  一般的な条件機構を使う．
  これらの中の提供された条件クラスのいずれかの生起は
  他の条件が何をシグナルしようと主張を満足する．

  \code{assertError()} はエラーを主張するための簡便関数である；
  これは \code{assertCondition()} を呼び出す．

  \code{assertWarning()} は警告が出ることを主張するが，
  エラーは\emph{しない}，他方で\cr
  \code{assertCondition(expr, "warning")} は警告にエラーが続いても
  満足される．例を見よ．
 }
\value{
  もし主張が満足されると，シグナルを出した全ての条件のリストが不可視で返される．
  これらのオブジェクトの解釈は \code{\link{conditionMessage}} を見よ．
  評価中にシグナルを出した\emph{全ての}条件が，
  それらが要求中に有るか無いかを問わず返されることを注意する．
}
\author{John Chambers と Martin Maechler}
\seealso{
  \code{\link{stop}}, \code{\link{warning}};
  \code{\link{signalCondition}}, \code{\link{tryCatch}}.
}
\examples{
  assertError(sqrt("abc"))
  assertWarning(matrix(1:8, 4,3))

  assertCondition( ""-1 ) # ok，全ての条件はこれを満足する

try( assertCondition(sqrt(2), "warning") )
## .. sqrt(2) の評価で警告を得ることに失敗
     assertCondition(sqrt("abc"), "error")   # ok
try( assertCondition(sqrt("abc"), "warning") )# -> error: 警告が出ない
     assertCondition(sqrt("abc"), "error")
  ## 上の assertError() 呼び出しと同一

assertCondition(matrix(1:5, 2,3), "warning")
try( assertCondition(matrix(1:8, 4,3), "error") )
## .. 期待されるエラーを得ることに失敗 ....

## 警告又はもっとまずいこと：
assertCondition(matrix(1:8, 4,3), "error","warning") # OK
assertCondition(matrix(1:8, 4, 3), "warning") # OK

## 両方がシグナルされる時：
ff <- function() { warning("my warning"); stop("my error") }
    assertCondition(ff(), "warning")
## しかし assertWarning はエラーが続くことを許さない
try(assertWarning(ff()))
    assertCondition(ff(), "error")          # ok
assertCondition(ff(), "error", "warning") # ok (黙って，警告をキャッチ)

## assertC..() が主張しないことを主張する[そして*一つ*の引数だけ]
assertCondition( assertCondition(sqrt( 2   ), "warning") )
assertCondition( assertCondition(sqrt("abc"), "warning"), "error")
assertCondition( assertCondition(matrix(1:8, 4,3), "error"),
                "error")
}
\keyword{programming}
\keyword{error}

\encoding{UTF8}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%\name{assertCondition}
%\alias{assertCondition}
%\alias{assertWarning}
%\alias{assertError}
%\title{Asserting Error Conditions}
%\description{
%  When testing code, it is not sufficient to check that results are correct,
%  but also that errors or warnings are signalled in appropriate
%  situations.  The functions described here provide a convenient
%  facility for doing so.  The three functions check that evaluating the
%  supplied expression produces an error, a warning or one of a
%  specified list of conditions, respectively.  If the assertion fails,
%  an error is signalled.
%}
%\usage{
%assertError(expr, verbose = FALSE)
%assertWarning(expr, verbose = FALSE)
%assertCondition(expr, ..., .exprString = , verbose = FALSE)
%}
%\arguments{
%  \item{expr}{an unevaluated \R expression which will be evaluated via
%    \code{\link{tryCatch}(expr, ..)}.}
%  \item{\dots}{\code{\link{character}} strings corresponding to the
%    classes of the conditions that would satisfy the assertion; e.g., \code{"error"} or
%    \code{"warning"}. If none are specified, any condition will
%    satisfy the assertion.  See the details section. }
%  \item{.exprString}{The string to be printed corresponding to
%    \code{expr}. By default, the actual \code{expr} will be
%    deparsed.  Will be omitted if the function is supplied with the
%    actual expression to be tested.  If \code{assertCondition()} is
%    called from another function, with the actual expression passed as
%    an argument to that function, supply the deparsed version.}
%  \item{verbose}{If \code{TRUE}, a message is printed when the
%    condition is satisfied.}
%}
%\details{
%  \code{assertCondition()} uses the general condition mechanism to
%  check all the conditions generated in evaluating \code{expr}.  The
%  occurrence of any of the supplied condition classes among these satisfies the
%  assertion regardless of what other conditions may be signalled.
%
%  \code{assertError()} is a convenience function for asserting errors;
%  it calls \code{assertCondition()}.
%
%  \code{assertWarning()} asserts that a warning will be signalled, but
%  \emph{not} an error, whereas \code{assertCondition(expr, "warning")}
%  will be satisfied even if an error follows the warning.  See the examples.
% }
%\value{
%  If the assertion is satisfied, a list of all the condition objects
%  signalled is returned, invisibly. See \code{\link{conditionMessage}} for the
%  interpretation of these objects.  Note that \emph{all} conditions
%  signalled during the evaluation are returned, whether or not they
%  were among the requirements.
%}
%\author{John Chambers and Martin Maechler}
%\seealso{
%  \code{\link{stop}}, \code{\link{warning}};
%  \code{\link{signalCondition}}, \code{\link{tryCatch}}.
%}
%\examples{
%  assertError(sqrt("abc"))
%  assertWarning(matrix(1:8, 4,3))
%
%  assertCondition( ""-1 ) # ok, any condition would satisfy this
%
%try( assertCondition(sqrt(2), "warning") )
%## .. Failed to get warning in evaluating sqrt(2)
%     assertCondition(sqrt("abc"), "error")   # ok
%try( assertCondition(sqrt("abc"), "warning") )# -> error: had no warning
%     assertCondition(sqrt("abc"), "error")
%  ## identical to assertError() call above
%
%assertCondition(matrix(1:5, 2,3), "warning")
%try( assertCondition(matrix(1:8, 4,3), "error") )
%## .. Failed to get expected error ....
%
%## either warning or worse:
%assertCondition(matrix(1:8, 4,3), "error","warning") # OK
%assertCondition(matrix(1:8, 4, 3), "warning") # OK
%
%## when both are signalled:
%ff <- function() { warning("my warning"); stop("my error") }
%    assertCondition(ff(), "warning")
%## but assertWarning does not allow an error to follow
%try(assertWarning(ff()))
%    assertCondition(ff(), "error")          # ok
%assertCondition(ff(), "error", "warning") # ok (quietly, catching warning)
%
%## assert that assertC..() does not assert [and use *one* argument only]
%assertCondition( assertCondition(sqrt( 2   ), "warning") )
%assertCondition( assertCondition(sqrt("abc"), "warning"), "error")
%assertCondition( assertCondition(matrix(1:8, 4,3), "error"),
%                "error")
%}
%\keyword{programming}
%\keyword{error}
%
%\encoding{UTF8}
%